const exporess = require('express');
const router = exporess.Router();
const bcrypt = require('bcryptjs');
const jwt = require('jsonwebtoken');
const keys = require('../../config/keys');

const { sendMail } = require('../../lib/mailer');

// load input validation
const validateRegisterInput = require('../../validation/register');
const validateLoginInput = require('../../validation/login');
const { validateRecoveryPasswordInput, validateRecoveryEmailInput } = require('../../validation/recovery');

const User = require('../../models/User');
const passport = require('passport');
const { setError } = require('../../lib/errors');
const { EXISTING_EMAIL, INVALID_CREDENTIALS, USER_NOT_FOUND, SERVER_ERROR } = require('../../constants/errors');

// @route   POST api/auth/register
// @desc    Register user
// @access  Public
router.post('/register', async (req, res) => {
  const { errors, isValid } = validateRegisterInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  try {
    const { email, password, name, surnames, commercialName, nif, phone, city, postalcode, address, account, logo } = req.body;
    const user = await User.findOne({ email });

    if (user) {
      errors.push(EXISTING_EMAIL);
      return res.status(400).json(errors);
    }

    const newUser = new User({ email, password, name, surnames, commercialName, nif, phone, city, postalcode, address, account, logo });

    const salt = await bcrypt.genSalt(10);

    newUser.password = await bcrypt.hash(password, salt);

    await newUser.save();

    const payload = { id: newUser.id, name: newUser.name, surnames: newUser.surnames };

    jwt.sign(payload, keys.secretOrKey, { expiresIn: 86400 }, (err, token) => {
      if (err) throw err;
      res.status(201).json(token);
    });
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   POST api/auth/login
// @desc    Login user. Return JWT
// @access  Public
router.post('/login', async (req, res) => {
  const { errors, isValid } = validateLoginInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { email, password } = req.body;

  try {
    const user = await User.findOne({ email })

    if (!user) {
      errors.push(setError(INVALID_CREDENTIALS));
      return res.status(400).json(errors);
    }

    const isMatch = await bcrypt.compare(password, user.password);

    if (!isMatch) {
      errors.push(setError(INVALID_CREDENTIALS));
      return res.status(400).json(errors);
    }

    const payload = { id: user.id, name: user.name, surnames: user.surnames };

    jwt.sign(payload, keys.secretOrKey, { expiresIn: 86400 }, (err, token) => {
      if (err) throw err;
      res.send(token);
    });
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   POST api/auth/sendjwt
// @desc    Send email to restore password
// @access  Public
router.post('/sendjwt', async (req, res) => {
  try {
    const { errors, isValid } = validateRecoveryEmailInput(req.body);

    if (!isValid) {
      return res.status(400).json(errors);
    }

    const { email } = req.body;

    const user = await User.findOne({ email });

    if (!user) {
      errors.push(setError(USER_NOT_FOUND));
      return res.status(404).json(errors);
    }

    const payload = { id: user.id, name: user.name, surnames: user.surnames };

    jwt.sign(payload, keys.secretOrKey, { expiresIn: 86400 }, (err, token) => {
      if (err) throw err;
      const htmlText = `Premeu el següent enllaç per <a href="${process.env.APP_URI}restore/${token}">restablir la contrasenya</a>.`;

      sendMail('info@easybill.com', email, "Password recovery", htmlText, (info) => {
        return res.send("Email sent");
      });
    });
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   POST api/auth/restore
// @desc    Restore user password
// @access  Private
router.put('/restore', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const { errors, isValid } = validateRecoveryPasswordInput(req.body);

    if (!isValid) {
      return res.status(400).json(errors);
    }

    const user = await User.findById(req.user.id);

    if (!user) {
      errors.push(setError(USER_NOT_FOUND));
      return res.status(404).json(errors);
    }

    const salt = await bcrypt.genSalt(10);

    user.password = await bcrypt.hash(req.body.password, salt);

    await user.save()

    delete user._doc.password;

    return res.status(201).json(user);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

module.exports = router;