const exporess = require('express');
const router = exporess.Router();

const validateCityInput = require('../../validation/city');

const User = require('../../models/User');
const Client = require('../../models/Client');
const City = require('../../models/City');

const passport = require('passport');
const { json, urlencoded } = require('body-parser');

// @route   GET api/cities
// @desc    Get all cities
// @access  Private
router.get('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const cities = await City.find({ user: req.user.id });

    return res.json(cities);
  } catch (error) {
    return res.status(500).send('Server error');
  }
});

// @route   GET api/cities/:id
// @desc    Get one city
// @access  Private
router.get('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const city = await City.findById(req.params.id);

    if (!city) {
      return res.status(404).send('City not found');
    }

    return res.json(city);
  } catch (error) {
    return res.status(500).send('Server error');
  }
});

// @route   POST api/cities
// @desc    Create city
// @access  Private
router.post('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateCityInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { code, name } = req.body;

  try {
    const existingCity = await City.findOne({ user: req.user.id, code, name });

    if (existingCity) {
      errors.duplicate = 'Combination of city and postal code already exists';
      return res.status(400).json(errors);
    }

    const city = new City({ user: req.user.id, code, name });

    await city.save();

    return res.status(201).json(city);
  } catch (error) {
    return res.status(500).send('Server error');
  }
});

// @route   PUT api/cities/:id
// @desc    Update city
// @access  Private
router.put('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateCityInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { code, name } = req.body;

  try {
    const city = await City.findById(req.params.id);

    if (!city) {
      errors.notfound = 'City not found';
      return res.status(404).json(errors);
    }

    city.name = name;
    city.code = code;

    await city.save();

    return res.status(202).json(city);
  } catch (error) {
    return res.status(500).send('Server error');
  }
});

// @route   DELETE api/cities':id
// @desc    Update city
// @access  Private
router.delete('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const city = await City.findById(req.params.id);

    if (!city) {
      return res.status(404).send('City not found');
    }

    const user = await User.find({ city: req.params.id });
    const client = await Client.find({ city: req.params.id });

    if (user.length || client.length) {
      return res.status(400).send('A user or a client are using this city');
    }

    await city.deleteOne();

    return res.status(204).send();
  } catch (error) {
    return res.status(500).send(error);
  }
});

module.exports = router;