const exporess = require('express');
const router = exporess.Router();

// load input validation
const validateClientrInput = require('../../validation/client');

const Client = require('../../models/Client');
const Invoice = require('../../models/Invoice');
const passport = require('passport');
const { setError } = require('../../lib/errors');

const { CLIENT_NOT_FOUND, SERVER_ERROR, EXISTING_NIF, CLIENT_HAS_INVOICES } = require('../../constants/errors');

// @route   GET api/clients
// @desc    Get all clients
// @access  Private
router.get('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const clients = await Client.find({ user: req.user.id });

    return res.json(clients);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   GET api/clients//:id
// @desc    Get client by id
// @access  Private
router.get('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];

  try {
    const client = await Client.findById(req.params.id);

    if (!client) {
      errors.push(setError(CLIENT_NOT_FOUND));
      return res.status(404).json(errors);
    }

    return res.json(client);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   GET api/clients/nif/:nif
// @desc    Get client by NIF
// @access  Private
router.get('/nif/:nif', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];

  try {
    const client = await Client.findOne({ user: req.user.id, nif: req.params.nif });

    if (!client) {
      errors.push(setError(CLIENT_NOT_FOUND));
      return res.status(404).json(errors);
    }

    return res.json(client);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   POST api/clients
// @desc    Add client
// @access  Private
router.post('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateClientrInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { name, surnames, email, commercialName, nif, phone, city, postalcode, address, account } = req.body;

  try {
    const existingClient = await Client.findOne({ user: req.user.id, nif });

    if (existingClient) {
      errors.push(setError(EXISTING_NIF));
      return res.status(400).json(errors);
    }

    const client = new Client({ user: req.user.id, name, surnames, email, commercialName, nif, phone, city, postalcode, address, account });

    await client.save();

    res.status(201).json(client);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   PUT api/clients/:id
// @desc    Update client
// @access  Private
router.put('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateClientrInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { name, surnames, email, commercialName, nif, phone, city, postalcode, address, account } = req.body;

  try {
    const client = await Client.findById(req.params.id);

    if (!client) {
      errors.push(setError(CLIENT_NOT_FOUND));
      return res.status(404).json(errors);
    }

    if (nif !== client.nif) {
      const duplicateNif = await Client.findOne({ nif });
      if (duplicateNif) {
        errors.push(setError(EXISTING_NIF));
        return res.status(400).json(errors);
      }
    }

    client.name = name;
    client.surnames = surnames;
    client.commercialName = commercialName;
    client.email = email;
    client.nif = nif;
    client.phone = phone;
    client.city = city;
    client.postalcode = postalcode;
    client.address = address;
    client.phone = phone;
    client.account = account;

    await client.save();

    return res.status(202).json(client);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   DELETE api/clients/:id
// @desc    Delete client
// @access  Private
router.delete('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];

  try {
    const client = await Client.findById(req.params.id);

    if (!client) {
      errors.push(setError(CLIENT_NOT_FOUND));
      return res.status(404).json(errors);
    }

    const invoices = await Invoice.find({ client: req.params.id });

    if (invoices.length) {
      errors.push(setError(CLIENT_HAS_INVOICES));
      return res.status(400).json(errors);
    }

    await client.deleteOne();

    return res.status(204).send();
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

module.exports = router;