const exporess = require('express');
const router = exporess.Router();

const { getInvoiceNumber, getDatePrefix, getInvoicePrefix } = require('../../lib/invoices');

// load input validation
const validateInvoiceInput = require('../../validation/invoice');

const Invoice = require('../../models/Invoice');
const EmittedInvoice = require('../../models/EmittedInvoice');
const Client = require('../../models/Client');
const Line = require('../../models/Line');

const passport = require('passport');
const User = require('../../models/User');
const { setError } = require('../../lib/errors');
const { INVOICE_NOT_FOUND, SERVER_ERROR, EMITTED_INVOICE_NOT_FOUND, CLIENT_NOT_FOUND, INVALID_DATE, INVOICE_CANNOT_BE_EMITTED, INVOICE_ALREADY_EMITTED, INVOICE_ALREADY_CANCELLED, INVOICE_CANNOT_BE_EDITED } = require('../../constants/errors');

// @route   GET api/invoices
// @desc    Get all invoices
// @access  Private
router.get('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const invoices = await Invoice.find({ user: req.user.id });

    return res.json(invoices);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   GET api/invoices/:number
// @desc    Get invoice by number
// @access  Private
router.get('/:number', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];

  try {
    const invoice = await Invoice.findOne({ user: req.user.id, number: req.params.number });

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    return res.json(invoice);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   GET api/invoices/emitted/:number
// @desc    Get emitted invoice by number
// @access  Private
router.get('/emitted/:number', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const errors = [];

    const invoice = await Invoice.findOne({ user: req.user.id, number: req.params.number });

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    const emittedInvoice = await EmittedInvoice.findOne({ invoice });

    if (!emittedInvoice) {
      errors.push(setError(EMITTED_INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    return res.json(emittedInvoice);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   POST api/invoices
// @desc    Add invoice
// @access  Private
router.post('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateInvoiceInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { client, name, ret } = req.body;

  try {
    const existingClient = await Client.findById(client);
    if (!existingClient) throw new Error();
  } catch (error) {
    errors.push(setError(CLIENT_NOT_FOUND));
    return res.status(404).json(errors);
}

  let date;

  if (req.body.date && req.body.date !== '') {
    try {
      date = new Date(req.body.date);
    } catch (error) {
      errors.push(setError(INVALID_DATE));
      return res.status(400).json(errors);
    }
  } else {
    date = new Date();
  }

  try {
    const invoices = await Invoice.find({ user: req.user.id });

    const number = getInvoiceNumber(date, invoices);

    const invoice = new Invoice({ user: req.user.id, client, date, number, name, ret, amount: 0, vat: 0 });

    await invoice.save();

    res.status(201).json(invoice);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   PUT api/invoices/emit/:id
// @desc    Emit invoice
// @put  Private
router.put('/emit/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];
  
  try {
    const invoice = await Invoice.findById(req.params.id);

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    if (invoice.cancelled) {
      errors.push(setError(INVOICE_CANNOT_BE_EMITTED));
      return res.status(400).json(errors);
    }

    if (invoice.emitted) {
      errors.push(setError(INVOICE_ALREADY_EMITTED));
      return res.status(400).json(errors);
    }

    const user = await User.findById(req.user.id);

    const client = await Client.findById(invoice.client);

    if (!client) {
      errors.push(setError(CLIENT_NOT_FOUND));
      return res.status(400).json(errors);
    }

    invoice.emitted = true;

    await invoice.save();

    const { date, number, name, ret, amount, vat } = invoice;

    const emittedInvoice = new EmittedInvoice({
      invoice, date, number, name, ret, amount, vat,
      user: {
        name: user.name,
        surnames: user.surnames,
        commercialName: user.commercialName,
        email: user.email,
        nif: user.nif,
        phone: user.phone,
        address: user.address,
        postalcode: user.postalcode,
        city: user.city,
        account: user.account,
        logo: user.logo
      },
      client: {
        name: client.name,
        surnames: client.surnames,
        commercialName: client.commercialName,
        email: client.email,
        nif: client.nif,
        phone: client.phone,
        address: client.address,
        postalcode: client.postalcode,
        city: client.city,
        account: client.account,
      }
    });

    await emittedInvoice.save();

    return res.status(202).json(invoice);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   PUT api/invoices/cancel/:id
// @desc    Cancel invoice
// @access  Private
router.put('/cancel/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const invoice = await Invoice.findById(req.params.id);

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    if (invoice.cancelled) {
      errors.push(setError(INVOICE_ALREADY_CANCELLED));
      return res.status(400).json(errors);
    }

    invoice.cancelled = true;

    await invoice.save();

    return res.status(202).json(invoice);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});


// @route   PUT api/invoices/:id
// @desc    Update invoice
// @access  Private
router.put('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateInvoiceInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { client, date, name, ret, charged, taxes } = req.body;

  try {
    const existingClient = await Client.findById(client);
    if (!existingClient) throw new Error();
  } catch (error) {
    errors.push(setError(CLIENT_NOT_FOUND));
    return res.status(400).json(errors);
  }

  try {
    const invoice = await Invoice.findById(req.params.id);

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    // cancelled invoices can not be edited
    if (invoice.cancelled) {
      errors.push(setError(INVOICE_CANNOT_BE_EDITED));
      return res.status(400).json(errors);
    }

    // change invoice number if invoice month is different from request date
    if (date && getDatePrefix(date) !== getInvoicePrefix(invoice)) {
      const invoices = await Invoice.find({ user: req.user.id });
      const number = getInvoiceNumber(date, invoices);
      invoice.number = number;
    }

    // Some emitted invoices values can not be changed
    if (!invoice.emitted) {
      invoice.client = client;
      invoice.date = date || new Date();
      invoice.name = name;
      invoice.ret = ret;
    }

    invoice.charged = charged;
    invoice.taxes = taxes;

    await invoice.save();

    res.status(202).json(invoice);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   DELETE api/invoices/:id
// @desc    Delete invoice
// @access  Private
router.delete('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];

  try {
    const invoice = await Invoice.findById(req.params.id);

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    if (invoice.emitted) {
      const emittedInvoice = await EmittedInvoice.findOne({ invoice });
      if (emittedInvoice) await emittedInvoice.deleteOne();
    }

    const lines = await Line.find({ invoice });

    lines.forEach(async (line) => await line.deleteOne());

    await invoice.deleteOne();

    return res.status(204).send();
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
})

module.exports = router;