const exporess = require('express');
const router = exporess.Router();

// load input validation
const validateLineInput = require('../../validation/line');

const Invoice = require('../../models/Invoice');
const Line = require('../../models/Line');

const passport = require('passport');
const { setError } = require('../../lib/errors');
const { SERVER_ERROR, LINE_NOT_FOUND, INVOICE_NOT_FOUND, INVOICE_CANNOT_BE_EDITED } = require('../../constants/errors');

const updateInvoice = async (invoice) => {
  try {
    const lines = await Line.find({ invoice: invoice._id });

    invoice.amount = lines.reduce((acc, line) => acc + line.amount * line.quantity, 0);
    invoice.vat = lines.reduce((acc, line) => acc + line.vat * line.amount * line.quantity, 0) / 100;

    invoice.save();
  } catch (error) {
    throw error;
  }
}

// @route   GET api/lines/:invoice_id
// @desc    Get all lines from invoice
// @access  Private
router.get('/:invoice_id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const lines = await Line.find({ invoice: req.params.invoice_id });

    return res.json(lines);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   GET api/lines/:invoice_id/:line_id
// @desc    Get a line
// @access  Private
router.get('/:invoice_id/:line_id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const errors = [];

    const line = await Line.findById(req.params.line_id);

    if (!line) {
      errors.push(setError(LINE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    return res.json(line);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   POST api/lines/:invoice_id
// @desc    Add a lines to invoice
// @access  Private
router.post('/:invoice_id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateLineInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { concept, quantity, amount, vat } = req.body;

  try {
    const invoice = await Invoice.findById(req.params.invoice_id);

    if (!invoice) {
      errors.push(setError(INVOICE_NOT_FOUND));
      return res.status(404).json(errors);
    }

    if (invoice.emitted || invoice.cancelled) {
      errors.push(setError(INVOICE_CANNOT_BE_EDITED))
      return res.status(400).json(errors);
    }

    const line = new Line({ invoice: req.params.invoice_id, concept, quantity, amount, vat });

    await line.save();

    updateInvoice(invoice);

    return res.status(201).json(line);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   PUT api/lines/:id
// @desc    Update a line
// @access  Private
router.put('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateLineInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  const { concept, quantity, amount, vat } = req.body;

  try {
    const line = await Line.findById(req.params.id);

    if (!line) {
      errors.push(setError(LINE_NOT_FOUND))
      return res.status(404).json(errors);
    }

    const invoice = await Invoice.findById(line.invoice);

    if (invoice.emitted || invoice.cancelled) {
      errors.push(setError(INVOICE_CANNOT_BE_EDITED));
      return res.status(400).json(errors);
    }

    line.concept = concept;
    line.quantity = quantity;
    line.amount = amount;
    line.vat = vat;

    await line.save();

    updateInvoice(invoice);

    return res.status(202).json(line);
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   DELETE api/lines/:id
// @desc    Delete a line
// @access  Private
router.delete('/:id', passport.authenticate('jwt', { session: false }), async (req, res) => {
  try {
    const errors = [];

    const line = await Line.findById(req.params.id);

    if (!line) {
      errors.push(setError(LINE_NOT_FOUND))
      return res.status(404).json(errors);
    }

    const invoice = await Invoice.findById(line.invoice);

    if (invoice.emitted || invoice.cancelled) {
      errors.push(setError(INVOICE_CANNOT_BE_EDITED));
      return res.status(400).json(errors);
    }

    await line.deleteOne();

    updateInvoice(invoice);

    return res.status(204).send();
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

module.exports = router;