const exporess = require('express');
const router = exporess.Router();
const bcrypt = require('bcryptjs');

// load input validation
const validateProfileInput = require('../../validation/profile');

const User = require('../../models/User');
const Client = require('../../models/Client');
const Invoice = require('../../models/Invoice');
const Line = require('../../models/Line');
const passport = require('passport');
const { EXISTING_EMAIL, USER_NOT_FOUND, SERVER_ERROR } = require('../../constants/errors');
const { setError } = require('../../lib/errors');

// @route   GET api/users
// @desc    Get user profile
// @access  Private
router.get('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];
  
  try {
    const user = await User.findById(req.user.id);

    if (!user) {
      errors.push(setError(USER_NOT_FOUND));
      return res.status(404).json(errors);
    }

    delete user._doc.password;

    return res.json(user._doc);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
})

// @route   PUT api/users
// @desc    Change user profile
// @access  Private
router.put('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const { errors, isValid } = validateProfileInput(req.body);

  if (!isValid) {
    return res.status(400).json(errors);
  }

  try {
    const user = await User.findById(req.user.id);

    if (!user) {
      errors.push(setError(USER_NOT_FOUND));
      return res.status(404).json(errors);
    }

    const { email, name, surnames, commercialName, nif, phone, city, postalcode, address, account, logo } = req.body;

    if (email !== user.email) {
      const userWhithSameEmail = await User.findOne({ email });
      if (userWhithSameEmail) {
        errors.push(setError(EXISTING_EMAIL));
        return res.status(400).json(errors);
      };
    }

    // user.password = await bcrypt.hash(password, salt);
    user.name = name;
    user.surnames = surnames;
    user.commercialName = commercialName;
    user.nif = nif;
    user.phone = phone;
    user.city = city;
    user.postalcode = postalcode;
    user.address = address;
    user.account = account;
    user.logo = logo;

    const salt = await bcrypt.genSalt(10);

    if (req.body.password) {
      user.password = await bcrypt.hash(req.body.password, salt);
    }

    await user.save();

    res.status(202).json(user);
  } catch (error) {
    console.log(error);
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

// @route   DELETE api/users
// @desc    Delete user
// @access  Private
router.delete('/', passport.authenticate('jwt', { session: false }), async (req, res) => {
  const errors = [];

  try {
    const user = await User.findById(req.user.id);

    if (!user) {
      errors.push(setError(USER_NOT_FOUND));
      return res.status(404).json(errors);
    }

    const clients = await Client.find({ user: req.user.id })
    const invoices = await Invoice.find({ user: req.user.id })

    await user.deleteOne();

    clients.forEach(async client => await client.deleteOne());

    invoices.forEach(async invoice => {
      const lines = await Line.find({ invoice: invoice.id })

      lines.forEach(async line => await line.deleteOne());

      await invoice.deleteOne()
    });

    await user.deleteOne();

    return res.status(204).send();
  } catch (error) {
    return res.status(500).json(setError(SERVER_ERROR));
  }
});

module.exports = router;